package net.javase.onenet.api;

import cn.hutool.http.ContentType;
import cn.hutool.http.Method;
import net.javase.onenet.enums.OneNetDomain;
import net.javase.onenet.enums.SignMethod;
import net.javase.onenet.model.base.BaseModel;
import net.javase.onenet.utils.RequestUtil;

import java.util.Map;

/**
 * ApiRequest
 *
 * @author Frank
 */
public class ApiRequest {

    /**
     * 参数组版本号
     */
    private String version;

    /**
     * 资源信息
     */
    private String res;

    /**
     * 接口api
     */
    private String api;

    /**
     * 指定请求方法
     */
    private Method method;

    /**
     * 指定请求头
     */
    private ContentType contentType;

    /**
     * domain
     */
    private OneNetDomain domain;

    /**
     * 签名方式
     */
    private SignMethod signMethod;

    /**
     * query参数
     */
    private Map<String, Object> query;

    /**
     * body参数
     */
    private Map<String, Object> body;

    /**
     * 发送请求
     *
     * @param accessKey accessKey
     * @return {@link ApiResponse}
     */
    public ApiResponse send(String accessKey) {
        return RequestUtil.doRequest(this, accessKey);
    }

    /**
     * 发送请求
     *
     * @param accessKey accessKey
     * @param clazz     clazz
     * @param <T>       T
     * @return T
     */
    public <T> T send(String accessKey, Class<T> clazz) {
        return RequestUtil.doRequest(this, accessKey, clazz);
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getRes() {
        return res;
    }

    public void setRes(String res) {
        this.res = res;
    }

    public String getApi() {
        return this.api;
    }

    public String url() {
        return this.api;
    }

    public void setApi(String api) {
        this.api = api;
    }

    public Method getMethod() {
        return this.method;
    }

    public void setMethod(Method method) {
        this.method = method;
    }

    public ContentType getContentType() {
        return this.contentType;
    }

    public void setContentType(ContentType contentType) {
        this.contentType = contentType;
    }

    public OneNetDomain getDomain() {
        return this.domain == null ? OneNetDomain.IOT_API : this.domain;
    }

    public void setDomain(OneNetDomain domain) {
        this.domain = domain;
    }

    public SignMethod getSignMethod() {
        return this.signMethod == null ? SignMethod.SHA256 : this.signMethod;
    }

    public void setSignMethod(SignMethod signMethod) {
        this.signMethod = signMethod;
    }

    public Map<String, Object> getQuery() {
        return this.query;
    }

    public void setQuery(Map<String, Object> query) {
        this.query = query;
    }

    public Map<String, Object> getBody() {
        return this.body;
    }

    public void setBody(Map<String, Object> body) {
        this.body = body;
    }

    ApiRequest(String version, String res, String api, Method method, ContentType contentType,
               OneNetDomain domain, SignMethod signMethod, Map<String, Object> query, Map<String, Object> body) {
        this.version = version;
        this.res = res;
        this.api = api;
        this.method = method;
        this.contentType = contentType;
        this.domain = domain;
        this.signMethod = signMethod;
        this.query = query;
        this.body = body;
    }

    public static ApiRequestBuilder builder() {
        return new ApiRequestBuilder();
    }

    public static class ApiRequestBuilder {

        /**
         * 参数组版本号
         */
        private String version = "2022-05-01";

        /**
         * 资源信息
         */
        private String res;
        private String api;
        private Method method;
        private ContentType contentType;
        private OneNetDomain domain;
        private SignMethod signMethod;
        private Map<String, Object> query;
        private Map<String, Object> body;

        ApiRequestBuilder() {
        }

        public ApiRequestBuilder version(String version) {
            this.version = version;
            return this;
        }

        public ApiRequestBuilder res(String res) {
            this.res = res;
            return this;
        }

        public ApiRequestBuilder api(String api) {
            this.api = api;
            return this;
        }

        public ApiRequestBuilder method(Method method) {
            this.method = method;
            return this;
        }

        public ApiRequestBuilder contentType(ContentType contentType) {
            this.contentType = contentType;
            return this;
        }

        public ApiRequestBuilder domain(OneNetDomain domain) {
            this.domain = domain;
            return this;
        }

        public ApiRequestBuilder signMethod(SignMethod signMethod) {
            this.signMethod = signMethod;
            return this;
        }

        public ApiRequestBuilder query(Map<String, Object> query) {
            this.query = query;
            return this;
        }

        public ApiRequestBuilder body(Map<String, Object> body) {
            this.body = body;
            return this;
        }

        public ApiRequestBuilder userid(String userid) {
            this.res = "userid/" + userid;
            return this;
        }

        /**
         * 设置请求参数
         *
         * @param model model
         * @param <T>   T
         * @return {@link ApiRequestBuilder}
         */
        public <T extends BaseModel> ApiRequestBuilder model(T model) {
            Map<String, Map<String, Object>> params = model.paramsMap();
            this.query = params.get("query");
            this.body = params.get("body");
            return this;
        }

        public ApiRequest build() {
            return new ApiRequest(this.version, this.res, this.api, this.method,
                    this.contentType, this.domain, this.signMethod, this.query, this.body);
        }

    }

}
