package net.javase.onenet.api;

import cn.hutool.http.ContentType;
import cn.hutool.http.Method;
import net.javase.onenet.config.Config;
import net.javase.onenet.enums.ApiEnum;
import net.javase.onenet.enums.SignMethod;
import net.javase.onenet.model.device.*;

import java.util.HashMap;
import java.util.Map;

/**
 * DeviceApi 设备API
 *
 * @author Frank
 */
public class DeviceApi {

    /**
     * 创建设备
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceCreate(Config config, net.javase.onenet.model.device.DeviceModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_CREATE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .model(model)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 编辑设备
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceUpdate(Config config, net.javase.onenet.model.device.DeviceModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_UPDATE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .model(model)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备详情
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceDetail(Config config, DeviceModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_DETAIL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 删除设备
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceDelete(Config config, DeviceModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_DELETE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 更新设备sec_key
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceResetSecKey(Config config, DeviceResetSecKeyModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_REST_SECKEY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备状态变更记录
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceStatusHistory(Config config, DeviceStatusHistoryModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_STATUS_HISTORY.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备事件记录查询
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceEventLog(Config config, DeviceEventLogModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_EVENT_LOG.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备操作记录查询
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceOperationLog(Config config, DeviceStatusHistoryModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_OPERATION_LOG.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备服务记录查询
     *
     * @param config  {@link Config}
     * @param traceId 调用服务后的request_id
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceServiceLog(Config config, String traceId) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("trace_id", traceId);
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_SERVICE_LOG.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备转移
     *
     * @param config {@link Config}
     * @param model  model
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceMove(Config config, DeviceMoveModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_MOVE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }


    /**
     * 设备分组列表
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupList(Config config, DeviceGroupListModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_LIST.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }


    /**
     * 设备分组详情
     *
     * @param config  {@link Config}
     * @param groupId 分组ID
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupDetail(Config config, String groupId) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("group_id", groupId);
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_DETAIL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 新建设备分组
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupCreate(Config config, DeviceGroupModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_CREATE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 编辑设备分组
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupUpdate(Config config, DeviceGroupModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_UPDATE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 删除设备分组
     *
     * @param config  {@link Config}
     * @param groupId 分组ID
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupDelete(Config config, String groupId) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("group_id", groupId);
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_DELETE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备分组-添加设备
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupAddDevices(Config config, DeviceGroupOptModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_ADD_DEVICES.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }


    /**
     * 设备分组-删除设备
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupDelDevices(Config config, DeviceGroupOptModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_DEL_DEVICES.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备分组-删除设备
     *
     * @param config {@link Config}
     * @param model  请求参数
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceGroupDevices(Config config, DeviceGroupListModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DEVICE_GROUP_DEL_DEVICES.api())
                .method(Method.GET)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

}
