package net.javase.onenet.api;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.ContentType;
import cn.hutool.http.Method;
import net.javase.onenet.config.Config;
import net.javase.onenet.enums.ApiEnum;
import net.javase.onenet.enums.SignMethod;
import net.javase.onenet.model.NbInstantModel;
import net.javase.onenet.model.NbOfflineModel;

import java.util.HashMap;
import java.util.Map;

/**
 * NbIotApi
 *
 * @author Frank
 */
public class NbIotApi {

    /**
     * 获取资源列表
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse resources(Config config, String imei) {
        return resources(config, imei, null);
    }

    /**
     * 获取资源列表
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @param objId  设备objectId，根据终端SDK确定
     * @return {@link ApiResponse}
     */
    public static ApiResponse resources(Config config, String imei, String objId) {
        Map<String, Object> query = new HashMap<>();
        query.put("imei", imei);
        if (StrUtil.isNotBlank(objId)) {
            query.put("obj_id", objId);
        }
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_RESOURCES.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 即时命令-资源发现
     *
     * @param config {@link Config}
     * @param model  {@link NbInstantModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse discover(Config config, NbInstantModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DISCOVER.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 即时命令-资源订阅
     *
     * @param config {@link Config}
     * @param model  {@link NbInstantModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse observe(Config config, NbInstantModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OBSERVE.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 即时命令-读设备资源
     *
     * @param config {@link Config}
     * @param model  {@link NbInstantModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse readResource(Config config, NbInstantModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 即时命令-写设备资源
     *
     * @param config {@link Config}
     * @param model  {@link NbInstantModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse writeResource(Config config, NbInstantModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .model(model)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 即时命令-即时命令下发
     *
     * @param config {@link Config}
     * @param model  {@link NbInstantModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse execute(Config config, NbInstantModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_EXECUTE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .model(model)
                .build()
                .send(config.getAccessKey());
    }


    // 缓存命令

    /**
     * 缓存命令-读设备资源
     *
     * @param config {@link Config}
     * @param model  {@link NbOfflineModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse readOfflineResource(Config config, NbOfflineModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 缓存命令-写设备资源
     *
     * @param config {@link Config}
     * @param model  {@link NbOfflineModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse writeOfflineResource(Config config, NbOfflineModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 缓存命令下发
     *
     * @param config {@link Config}
     * @param model  {@link NbOfflineModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse executeOffline(Config config, NbOfflineModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE_EXECUTE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .model(model)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 查看缓存命令详情
     *
     * @param config {@link Config}
     * @param uuid   缓存命令ID
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse offlineHistory(Config config, String uuid, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE_HISTORY_UUID.api(uuid))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 取消缓存命令
     *
     * @param config {@link Config}
     * @param uuid   缓存命令ID
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse offlineCancel(Config config, String uuid, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE_CANCEL_UUID.api(uuid))
                .method(Method.PUT)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 取消所有未下发缓存命令
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse offlineCancelAll(Config config, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE_CANCEL_ALL.api())
                .method(Method.PUT)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 全链路日志查询
     *
     * @param config {@link Config}
     * @param uuid   缓存命令ID
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse offlineHistoryPiecewise(Config config, String uuid, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_OFFLINE_HISTORY_PIECEWISE.api(uuid))
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 查看bs_psk
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse devicePskQuery(Config config, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DEVICE_PSK.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 更新bs_psk
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @param key    修改连接引导机的PSK，长度8-16个字节，字母或数字组成
     * @return {@link ApiResponse}
     */
    public static ApiResponse devicePskUpdate(Config config, String imei, String key) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        Map<String, Object> body = new HashMap<>((int) ((1 / 0.75f) + 1));
        body.put("key", key);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DEVICE_PSK.api())
                .method(Method.PUT)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .body(body)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 查看acc_psk
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceAccPskQuery(Config config, String imei) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DEVICE_ACCPSK.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 新增acc_psk
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @param key    修改连接引导机的PSK，长度8-16个字节，字母或数字组成
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceAccPskAdd(Config config, String imei, String key) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        Map<String, Object> body = new HashMap<>((int) ((1 / 0.75f) + 1));
        body.put("key", key);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DEVICE_ACCPSK.api())
                .method(Method.POST)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .body(body)
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 更新acc_psk
     *
     * @param config {@link Config}
     * @param imei   nbiot设备的身份码
     * @param key    修改连接引导机的PSK，长度8-16个字节，字母或数字组成
     * @return {@link ApiResponse}
     */
    public static ApiResponse deviceAccPskUpdate(Config config, String imei, String key) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("imei", imei);
        Map<String, Object> body = new HashMap<>((int) ((1 / 0.75f) + 1));
        body.put("key", key);
        return ApiRequest.builder()
                .api(ApiEnum.NBIOT_DEVICE_ACCPSK.api())
                .method(Method.PUT)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .query(query)
                .body(body)
                .build()
                .send(config.getAccessKey());
    }

}
