package net.javase.onenet.api;

import cn.hutool.http.ContentType;
import cn.hutool.http.Method;
import net.javase.onenet.config.Config;
import net.javase.onenet.enums.ApiEnum;
import net.javase.onenet.enums.SignMethod;
import net.javase.onenet.model.tsl.ThingModel;
import net.javase.onenet.model.tsl.ThingQueryHistoryModel;
import net.javase.onenet.model.tsl.ThingQueryModel;

import java.util.HashMap;
import java.util.Map;

/**
 * TSLApi 物模型API
 *
 * @author Frank
 */
public class TSLApi {

    /**
     * 物模型系统功能点列表
     *
     * @param config {@link Config}
     * @return {@link ApiResponse}
     */
    public static ApiResponse querySystemThingModel(Config config) {
        return ApiRequest.builder()
                .api(ApiEnum.TSL_QUERY_SYSTEM_THING_MODEL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 物模型查询
     *
     * @param config    {@link Config}
     * @param productId 产品ID
     * @return {@link ApiResponse}
     */
    public static ApiResponse queryThingModel(Config config, String productId) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("product_id", productId);
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_THING_MODEL.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设置设备属性
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse setDeviceProperty(Config config, ThingModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.SET_DEVICE_PROPERTY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 获取设备属性详情
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse queryDevicePropertyDetail(Config config, ThingQueryModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_DEVICE_PROPERTY_DETAIL.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备属性最新数据查询
     *
     * @param config     {@link Config}
     * @param productId  产品ID
     * @param deviceName 设备名称
     * @return {@link ApiResponse}
     */
    public static ApiResponse queryDeviceProperty(Config config,
                                                  String productId, String deviceName) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("product_id", productId);
        query.put("device_name", deviceName);
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_DEVICE_PROPERTY.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备属性最新数据查询
     *
     * @param config     {@link Config}
     * @param productId  产品ID
     * @param deviceName 设备名称
     * @param clazz      clazz
     * @param <T>        T
     * @return T
     */
    public static <T> T queryDeviceProperty(Config config, String productId, String deviceName, Class<T> clazz) {
        Map<String, Object> query = new HashMap<>((int) ((1 / 0.75f) + 1));
        query.put("product_id", productId);
        query.put("device_name", deviceName);
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_DEVICE_PROPERTY.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(query)
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey(), clazz);
    }

    /**
     * 设备属性最新数据查询
     *
     * @param config {@link Config}
     * @param model  {@link ThingQueryHistoryModel}
     * @return {@link ApiResponse}
     */
    public static ApiResponse queryDevicePropertyHistory(Config config, ThingQueryHistoryModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_DEVICE_PROPERTY_HISTORY.api())
                .method(Method.GET)
                .signMethod(SignMethod.MD5)
                .query(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备服务调用
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse callService(Config config, ThingModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.CALL_SERVICE.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备属性期望设置
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse setDeviceDesiredProperty(Config config, ThingModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.SET_DEVICE_DESIRED_PROPERTY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备属性期望查询
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse queryDeviceDesiredProperty(Config config, ThingQueryModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.QUERY_DEVICE_DESIRED_PROPERTY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

    /**
     * 设备属性期望删除
     *
     * @param config {@link Config}
     * @param model  请求参数对象
     * @return {@link ApiResponse}
     */
    public static ApiResponse delDeviceDesiredProperty(Config config, ThingModel model) {
        return ApiRequest.builder()
                .api(ApiEnum.DELETE_DEVICE_DESIRED_PROPERTY.api())
                .method(Method.POST)
                .contentType(ContentType.JSON)
                .signMethod(SignMethod.MD5)
                .body(model.toMap())
                .userid(config.getUserid())
                .build()
                .send(config.getAccessKey());
    }

}
