package net.javase.onenet.utils;

import cn.hutool.core.util.URLUtil;
import net.javase.onenet.enums.SignMethod;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;

/**
 * SignUtil 签名工具
 *
 * @author Frank
 */
public class SignUtil {

    /**
     * 默认版本号
     */
    public static final String DEFAULT_VERSION = "2022-05-01";

    /**
     * 构建token
     *
     * @param version        version
     * @param resourceName   resourceName
     * @param expirationTime expirationTime
     * @param signMethod     signMethod
     * @param accessKey      accessKey
     * @return token {@link String}
     */
    public static String assembleToken(String version,
                                       String resourceName,
                                       String expirationTime,
                                       SignMethod signMethod,
                                       String accessKey) {
        StringBuilder sb = new StringBuilder();
        String res = URLUtil.encodeAll(resourceName, StandardCharsets.UTF_8);
        String sign = generatorSignature(version, resourceName, expirationTime, accessKey, signMethod.name().toLowerCase());
        sb.append("version=").append(version)
                .append("&res=").append(res)
                .append("&et=").append(expirationTime)
                .append("&method=").append(signMethod.name().toLowerCase())
                .append("&sign=").append(URLUtil.encodeAll(sign, StandardCharsets.UTF_8));
        return sb.toString();
    }

    /**
     * 生成sign
     *
     * @param version         version
     * @param resourceName    resourceName
     * @param expirationTime  expirationTime
     * @param accessKey       accessKey
     * @param signatureMethod signatureMethod
     * @return sign
     */
    public static String generatorSignature(String version,
                                            String resourceName,
                                            String expirationTime,
                                            String accessKey,
                                            String signatureMethod) {
        String encryptText = expirationTime + "\n" + signatureMethod + "\n" + resourceName + "\n" + version;
        byte[] bytes = hmacEncrypt(encryptText, accessKey, signatureMethod);
        return Base64.getEncoder().encodeToString(bytes);
    }

    /**
     * 加密
     *
     * @param data            data
     * @param key             key
     * @param signatureMethod signatureMethod
     * @return 加密后数据
     */
    public static byte[] hmacEncrypt(String data, String key, String signatureMethod) {
        String instanceName = "Hmac" + signatureMethod.toUpperCase();
        SecretKeySpec secretKey = new SecretKeySpec(Base64.getDecoder().decode(key), instanceName);
        try {
            Mac mac = Mac.getInstance(instanceName);
            mac.init(secretKey);
            return mac.doFinal(data.getBytes(StandardCharsets.UTF_8));
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("不支持的签名算法", e);
        } catch (InvalidKeyException e) {
            throw new RuntimeException("无效秘钥", e);
        }
    }

}
